import { BoxPanel } from '@lumino/widgets';
import { SessionContext, Toolbar, ToolbarButton } from '@jupyterlab/apputils';
import { CodeCellModel, CodeCell } from '@jupyterlab/cells';
import { CodeMirrorMimeTypeService } from '@jupyterlab/codemirror';
import { runIcon } from '@jupyterlab/ui-components';
import { CompleterModel, Completer, CompletionHandler, KernelConnector } from '@jupyterlab/completer';
import { RenderMimeRegistry, standardRendererFactories as initialFactories } from '@jupyterlab/rendermime';
import { SessionManager, KernelManager, KernelSpecManager } from '@jupyterlab/services';
import { ServerConnection } from '@jupyterlab/services';
import { CommandRegistry } from '@lumino/commands';
import KernelModel from './KernelModel';
import { useDispatch } from "react-redux";
import { map } from "rxjs/operators";
import { cellEpics, cellActions, cellReducer } from './CellRedux';
import store from '../../redux/Store';
import { asObservable } from '../../jupyter/LuminoObservable';
class CellLumino {
    constructor(source) {
        this.execute = () => {
            CodeCell.execute(this.codeCell, this.sessionContext);
        };
        this.cellPanel = new BoxPanel();
        this.cellPanel.direction = 'top-to-bottom';
        this.cellPanel.spacing = 0;
        const serverSettings = ServerConnection.makeSettings();
        const kernelManager = new KernelManager({
            serverSettings
        });
        const specsManager = new KernelSpecManager({
            serverSettings
        });
        const sessionManager = new SessionManager({
            serverSettings,
            kernelManager
        });
        this.sessionContext = new SessionContext({
            sessionManager,
            specsManager,
            name: 'Datalayer'
        });
        const mimeService = new CodeMirrorMimeTypeService();
        // Initialize the command registry with the bindings.
        const commands = new CommandRegistry();
        const useCapture = true;
        // Setup the keydown listener for the document.
        document.addEventListener('keydown', event => {
            commands.processKeydownEvent(event);
        }, useCapture);
        // Create the cell widget with a default rendermime instance.
        const rendermime = new RenderMimeRegistry({ initialFactories });
        const codeCell = new CodeCell({
            rendermime,
            model: new CodeCellModel({
                cell: {
                    cell_type: 'code',
                    source: source,
                    metadata: {}
                }
            })
        });
        this.codeCell = codeCell.initializeState();
        // Handle the mimeType for the current kernel asynchronously.
        this.sessionContext.kernelChanged.connect(() => {
            var _a, _b;
            void ((_b = (_a = this.sessionContext.session) === null || _a === void 0 ? void 0 : _a.kernel) === null || _b === void 0 ? void 0 : _b.info.then(info => {
                const lang = info.language_info;
                const mimeType = mimeService.getMimeTypeByLanguage(lang);
                this.codeCell.model.mimeType = mimeType;
            }));
        });
        // Use the default kernel.
        this.sessionContext.kernelPreference = { autoStartDefault: true };
        // Set up a completer.
        const editor = this.codeCell.editor;
        const model = new CompleterModel();
        const completer = new Completer({ editor, model });
        const connector = new KernelConnector({ session: this.sessionContext.session });
        const handler = new CompletionHandler({ completer, connector });
        // Set the handler's editor.
        handler.editor = editor;
        // Hide the widget when it first loads.
        completer.hide();
        // Create a toolbar for the cell.
        const toolbar = new Toolbar();
        toolbar.addItem('spacer', Toolbar.createSpacerItem());
        const runButton = new ToolbarButton({
            icon: runIcon,
            onClick: () => {
                CodeCell.execute(this.codeCell, this.sessionContext);
            },
            tooltip: 'Run'
        });
        toolbar.addItem('run', runButton);
        toolbar.addItem('interrupt', Toolbar.createInterruptButton(this.sessionContext));
        toolbar.addItem('restart', Toolbar.createRestartButton(this.sessionContext));
        // toolbar.addItem('name', Toolbar.createKernelNameItem(this.sessionContext));
        toolbar.addItem('status', Toolbar.createKernelStatusItem(this.sessionContext));
        this.cellPanel.addWidget(completer);
        this.cellPanel.addWidget(toolbar);
        BoxPanel.setStretch(toolbar, 0);
        this.cellPanel.addWidget(this.codeCell);
        BoxPanel.setStretch(this.codeCell, 1);
        // Handle widget state.
        window.addEventListener('resize', () => {
            this.cellPanel.update();
        });
        this.codeCell.outputsScrolled = false;
        this.codeCell.activate();
        // Add the commands.
        commands.addCommand('invoke:completer', {
            execute: () => {
                handler.invoke();
            }
        });
        commands.addCommand('run:cell', {
            execute: () => CodeCell.execute(this.codeCell, this.sessionContext)
        });
        commands.addKeyBinding({
            selector: '.jp-InputArea-editor.jp-mod-completer-enabled',
            keys: ['Tab'],
            command: 'invoke:completer'
        });
        commands.addKeyBinding({
            selector: '.jp-InputArea-editor',
            keys: ['Shift Enter'],
            command: 'run:cell'
        });
        // Start up the kernel.
        this.sessionContext.initialize().then(() => {
            const kernelModel = new KernelModel(this.sessionContext);
            kernelModel.execute(source);
            void CodeCell.execute(this.codeCell, this.sessionContext);
            /*
            executePromise.then((msg: void | KernelMessage.IExecuteReplyMsg) => {
                console.log('Kernel messsage', msg);
                dispatch(tellActions.update.started({
                  kernelAvailable: true,
                }));
            })
            */
        });
        const dispatch = useDispatch();
        const outputs$ = asObservable(codeCell.outputArea.outputLengthChanged);
        outputs$.subscribe(n => { dispatch(cellActions.outputs.started(n)); });
        outputs$.pipe(map(output => console.log('---', output)));
        store.injectReducer('cell', cellReducer);
        store.injectEpic(cellEpics(this));
    }
    get panel() {
        return this.cellPanel;
    }
    get cell() {
        return this.codeCell;
    }
}
export default CellLumino;
//# sourceMappingURL=CellLumino.js.map