import { CommandRegistry } from '@lumino/commands';
import { CommandPalette, SplitPanel } from '@lumino/widgets';
import { ServiceManager } from '@jupyterlab/services';
import { MathJaxTypesetter } from '@jupyterlab/mathjax2';
import { PageConfig, URLExt } from '@jupyterlab/coreutils';
import { NotebookModelFactory, NotebookPanel, NotebookWidgetFactory } from '@jupyterlab/notebook';
import { Completer, CompleterModel, CompletionHandler, KernelConnector } from '@jupyterlab/completer';
import { editorServices } from '@jupyterlab/codemirror';
import { DocumentManager } from '@jupyterlab/docmanager';
import { DocumentRegistry } from '@jupyterlab/docregistry';
import { standardRendererFactories as initialFactories, RenderMimeRegistry } from '@jupyterlab/rendermime';
import { setupCommands } from './CellsCommands';
import { ProviderMock, WebSocketProviderWithLocks } from '@jupyterlab/docprovider';
import { ServerConnection } from '@jupyterlab/services';
class CellsLumino {
    constructor() {
        this.notebookPanel = new SplitPanel();
        this.notebookPanel.id = 'dla-jlab-notebook';
        this.notebookPanel.spacing = 0;
        const manager = new ServiceManager();
        void manager.ready.then(() => {
            createApp(manager, this.notebookPanel);
        });
        function createApp(manager, panel) {
            // Initialize the command registry with the bindings.
            const commands = new CommandRegistry();
            const useCapture = true;
            // Setup the keydown listener for the document.
            document.addEventListener('keydown', event => {
                commands.processKeydownEvent(event);
            }, useCapture);
            const rendermime = new RenderMimeRegistry({
                initialFactories: initialFactories,
                latexTypesetter: new MathJaxTypesetter({
                    url: PageConfig.getOption('mathjaxUrl'),
                    config: PageConfig.getOption('mathjaxConfig')
                })
            });
            const opener = {
                open: (widget) => {
                    // Do nothing for sibling widgets for now.
                }
            };
            const server = ServerConnection.makeSettings();
            const url = URLExt.join(server.wsUrl, 'api/yjs');
            const collaborative = PageConfig.getOption('collaborative') == 'true' ? true : false;
            const docProviderFactory = (options) => {
                return collaborative
                    ? new WebSocketProviderWithLocks(Object.assign(Object.assign({}, options), { url }))
                    : new ProviderMock();
            };
            const docRegistry = new DocumentRegistry();
            const docManager = new DocumentManager({
                registry: docRegistry,
                manager,
                opener,
                collaborative,
                docProviderFactory
            });
            const mFactory = new NotebookModelFactory({});
            const editorFactory = editorServices.factoryService.newInlineEditor;
            const contentFactory = new NotebookPanel.ContentFactory({ editorFactory });
            const wFactory = new NotebookWidgetFactory({
                name: 'Notebook',
                modelName: 'notebook',
                fileTypes: ['notebook'],
                defaultFor: ['notebook'],
                preferKernel: true,
                canStartKernel: true,
                rendermime,
                contentFactory,
                mimeTypeService: editorServices.mimeTypeService
            });
            docRegistry.addModelFactory(mFactory);
            docRegistry.addWidgetFactory(wFactory);
            const notebookPath = 'test.ipynb';
            const nbWidget = docManager.open(notebookPath);
            const palette = new CommandPalette({ commands });
            palette.addClass('notebookCommandPalette');
            const editor = nbWidget.content.activeCell && nbWidget.content.activeCell.editor;
            const model = new CompleterModel();
            const completer = new Completer({ editor, model });
            const sessionContext = nbWidget.context.sessionContext;
            const connector = new KernelConnector({
                session: sessionContext.session
            });
            const handler = new CompletionHandler({ completer, connector });
            void sessionContext.ready.then(() => {
                handler.connector = new KernelConnector({
                    session: sessionContext.session
                });
            });
            // Set the handler's editor.
            handler.editor = editor;
            // Listen for active cell changes.
            nbWidget.content.activeCellChanged.connect((sender, cell) => {
                handler.editor = cell && cell.editor;
            });
            // Hide the widget when it first loads.
            completer.hide();
            SplitPanel.setStretch(palette, 0);
            SplitPanel.setStretch(nbWidget, 1);
            panel.addWidget(palette);
            panel.addWidget(nbWidget);
            // Handle resize events.
            window.addEventListener('resize', () => {
                panel.update();
            });
            setupCommands(commands, palette, nbWidget, handler);
        }
    }
    get panel() {
        return this.notebookPanel;
    }
}
export default CellsLumino;
//# sourceMappingURL=CellsLumino.js.map